//#define DEBUG 1

#ifdef DEBUG

//Serial.print(" = ");
//Serial.println();
#endif

// ARDUINO UNO COM4 ********************************************************************************************************************************************************

// ARDUINO STEPR (URBIUM SEQUENCER) CODE 1.0 - March 2nd 2018 - Miroslaw Sowa

// ver 1.1 - This version has a steady working ISR but I need the next one ver 1.2 to have a ISR for BPM timing.
// ver 1.2 - Removed timer CTC interrupt code from www.engblaze.com
//         - Installed uClock library for BPM timing ISR
//         - Started separating the display code
// ver 1.3 -
// ver 1.4 - added SPI library for the MCP4921 DAC SPI voltage out IC
// ver.1.6 - starting from 1.4 again because of the dead end ver 1.5. Too many bugs, Dead end decided to stop troubleshooting it and decided to start from skratch v1.8
// ver.1.7 - not finished, was started to implement the TTP229 TOUCH SENSING for the top plate.
// ver.1.8 - Starting from skratch
//
////////////////////////////////////////////////////////////////////////////////////// TO DO LIST; /////////////////////////////////////////////////////////////////////////
//
// [ ] CLOCK IN FEATURE ACCEPTING TEMPO FROM OUTSIDE CLOCK SOURCE 5VOLT PULSES
//
// [ ] GLIDE FEATURE IMPLEMENTING
//
// [ ] FEATURE THAT SENDS CV VALUES ONLY DURING WHEN GATE IS HIGH
//
// [ ] ADD FAST DIGITAL OUT CODE IMPROVEMENT
//
// [ ] MINIMIZE THE TIME THE CALCULATIONS ARE MADE EVERY LOOP, MAKE TEMP0... CALUCLATE ONLY WHEN IT CHANGES
//

// [ ] FIX MIDI NOTE OFF UNNESESSARY MESSAGES

// [ ] CODE SO EEPROM IS READ ONLY IF IT WAS WRITTEN TO PREVIOUSLY

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//  Voltage from DAC that is 0-5 volt range. This will have a range of 5 octaves (1volt/octave) and will start with MIDI note 21 (A0, 27.5Hz) at 0volts, MIDI note 81 (A5, 880Hz) at 5volts.

// FAST DIGITAL READ & WRITE
// SPI FOR DAC
// UCLOCK FOR TIMING

#include <EEPROM.h>

#include "SPI.h" // necessary library
word outputValue = 0; // a word is a 16-bit number
byte data = 0; // and a byte is an 8-bit number
const char SS_PIN = 10; // SS pin for SPI

//https://github.com/midilab/uClock
#include "uClock.h"

const char  GATEOUT = A2; // Gate Output pin

#include <LiquidCrystalFast.h>                                            // DISPLAY
const char rs = A0, rw = 6, en = A3, d4 = 5, d5 = 4, d6 = 3, d7 = 2;      // DISPLAY
LiquidCrystalFast lcd(rs, rw, en, d4, d5, d6, d7);                        // DISPLAY

// 4051 Multiplexer A,B,C channels
const int channel[] = {7, 8, 9};
byte arrayChanA[]   = {0, 1, 0, 1, 0, 1, 0, 1};
byte arrayChanB[]   = {0, 0, 1, 1, 0, 0, 1, 1};
byte arrayChanC[]   = {0, 0, 0, 0, 1, 1, 1, 1};

//the input pin - mux input DP3T
const char inputPinDP3T = 12; //
//the output pin - mux output LEDS
const char outputPinLEDS = A1;
//the input pin - mux input POTS   ******************************* CHANGE FOR AVR *************** A6
const char inputPinPOTS = A4;  //A4 = 18
//the input pin - mux input TACTS  ******************************* CHANGE FOR AVR *************** A7
const char inputPinTACTS = A5; //A5 = 19

char arrayDP3T[] =              {0, 0, 0, 0, 0, 0, 0, 0}; // values of the 2 triple switches and up and down buttons
char arrayDP3Tprevious[] =      {0, 0, 0, 0, 0, 0, 0, 0}; // to debounce the tact switches and DP3T swiches
char arrayTACTS[] =             {0, 0, 0, 0, 0, 0, 0, 0}; // values of 8 tact switches (1 for HIGH and 0 for LOW)
char arrayTACTSprevious[] =     {0, 0, 0, 0, 0, 0, 0, 0}; // values of 8 tact switches (1 for HIGH and 0 for LOW)
int  arrayPOTS[] =              {0, 0, 0, 0, 0, 0, 0, 0}; // values of 8 potentiometers in 10bit resolution (0-1023)
int  arrayPOTSprevious[] =      {0, 0, 0, 0, 0, 0, 0, 0}; // values of 8 potentiometers in 10bit resolution (0-1023)

char arrayPULSEcount[] =        {1, 1, 1, 1, 1, 1, 1, 1}; // values for PULSE COUNT from 1 to 8
char arrayGATEmode[] =          {1, 1, 1, 1, 1, 1, 1, 1}; // values for GATE MODE from 0 to 3 (0 for mute / 1 for 1 gate trigger for 1st note only / 2 for 1 gate trigger for each PULSE COUNT / 3 for 1 gate trigger for all PULSE COUNTs)
char arraySKIPglide[] =         {0, 0, 0, 0, 0, 0, 0, 0}; // values for SKIP/GLIDE/RATCHET from -3 to 2 (-3 for RATCHET 4, -2 for RATCHET 3, -1 for RATCHET 2, 0 for no SKIP no SLIDE no RATCHET, 1 for GLIDE, 2 for SKIP)
char arraySKIPglidePREVIOUS[] = {0, 0, 0, 0, 0, 0, 0, 0}; // values for SKIP/GLIDE/RATCHET from -3 to 2 (-3 for RATCHET 4, -2 for RATCHET 3, -1 for RATCHET 2, 0 for no SKIP no SLIDE no RATCHET, 1 for GLIDE, 2 for SKIP)

byte arrayMIDInote[] =          {0, 0, 0, 0, 0, 0, 0, 0};

int  arrayDISPLAY1[] =          {0, 0, 0, 0, 0, 0, 0, 0}; // values of the 1st line of the dislay ( 1 to 8 for each STEP )
int  arrayDISPLAY1previous[] =  {0, 0, 0, 0, 0, 0, 0, 0};
int  arrayDISPLAY2[] =          {0, 0, 0, 0, 0, 0, 0, 0}; // values of the 2nd line of the dislay ( 1 to 8 for each STEP )
int  arrayDISPLAY2previous[] =  {0, 0, 0, 0, 0, 0, 0, 0};
char arrayDISPLAY3[] =          {0, 0, 0, 0, 0, 0, 0, 0}; // values of the LEDS to be lit ( 1 to 8 for each STEP )
char arrayDISPLAY4[] =          {0, 0, 0, 0, 0, 0, 0, 0}; // values for SKIP/GLIDE/RATCHET ( 1 to 8 for each STEP )
char arrayDISPLAY4previous[] =  {0, 0, 0, 0, 0, 0, 0, 0};

//int arrayCVvalues[] =         {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,68,137,205,273,341,410,478,546,614,683,751,819,887,956,1024,1092,1160,1229,1297,1365,1433,1502,1570,1638,1706,1775,1843,1911,1979,2048,2116,2184,2252,2321,2389,2457,2525,2594,2662,2730,2798,2867,2935,3003,3071,3140,3208,3276,3344,3413,3481,3549,3617,3686,3754,3822,3890,3959,4027,4095,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } ; // 61 CV values out of 128
int arrayCVvalues[] =           {0, 68, 137, 205, 273, 341, 410, 478, 546, 614, 683, 751, 819, 887, 956, 1024, 1092, 1160, 1229, 1297, 1365, 1433, 1502, 1570, 1638, 1706, 1775, 1843, 1911, 1979, 2048, 2116, 2184, 2252, 2321, 2389, 2457, 2525, 2594, 2662, 2730, 2798, 2867, 2935, 3003, 3071, 3140, 3208, 3276, 3344, 3413, 3481, 3549, 3617, 3686, 3754, 3822, 3890, 3959, 4027, 4095} ; // 61 CV values
//int arrayCVoltPERoctave[] =   {0,85,171,256,341,427,512,597,683,768,853,939,1024,1109,1195,1280,1365,1451,1536,1621,1707,1792,1877,1963,2048,2133,2219,2304,2389,2475,2560,2645,2731,2816,2901,2987,3072,3157,3243,3328,3413,3499,3584,3669,3755,3840,3925,4011,4096 } ; // 48 values, 1 volt per octave, 4 octaves total
byte arraySCALEchromatic[] =    {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127};
//byte arraySCALEmajor[] =      {0, 0, 2, 2, 4, 5, 5, 7, 7, 9,  9, 11, 12, 12, 14, 14, 16, 17, 17, 19, 19, 21, 21, 23, 24, 24, 26, 26, 28, 29, 29, 31, 31, 33, 33, 35, 36, 36, 38, 38, 40, 41, 41, 43, 43, 45, 45, 47, 48, 48, 50, 50, 52, 53, 53, 55, 55, 57, 57, 59, 60, 60, 62, 62, 64, 65, 65, 67, 68, 68, 70, 70,72,72,74,75,75,77,77,79,80,80,82,82,84,84,86,87,87,89,89,91,92,92,94,94,96,96,98,99,99,101,101,103,104,104,106,106,108};
//byte arraySCALEmajor[] =      {0, 2, 4, 5, 7, 9, 11, 12, 14, 16, 17, 19, 21, 23, 24, 26, 28, 29, 31, 33, 35, 36, 38, 40, 41, 43, 45, 47, 48, 50, 52, 53, 55, 57, 59, 60, 62, 64, 65, 67, 68, 70, 72, 74, 75, 77, 79, 80, 82, 84, 86, 87, 89, 91, 92, 94, 96, 98, 99, 101, 103, 104, 106, 108};
//byte arraySCALEminor[] =      {0, 2, 3, 5, 7, 8, 10, 12, 14, 15, 17, 19, 20, 22, 24, 26, 27, 29, 31, 32, 34, 36, 38, 39, 41, 43, 44, 46, 48, 50, 51, 53, 55, 56, 58, 60, 62, 63, 65, 67, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114};
byte arraySCALEdorian[] =       {0, 0, 2, 3, 4, 5, 5, 7, 7, 9, 10, 10, 12, 12, 14, 15, 15, 17, 17, 19, 19, 21, 22, 22, 24, 24, 26, 27, 27, 29, 29, 31, 31, 33, 34, 34, 36, 36, 38, 39, 39, 41, 41, 43, 43, 45, 46, 46, 48, 48, 50, 51, 51, 53, 53, 55, 55, 57, 58, 58, 60, 60, 62, 63, 63, 65, 65, 67, 67, 69, 70, 70, 72, 72, 74, 75, 75, 77, 77, 79, 79, 81, 82, 82, 84, 84, 86, 87, 87, 89, 89, 91, 91, 93, 94, 94, 96, 96, 98, 99, 99, 101, 101, 103, 103, 105, 106, 106, 108, 110, 110, 111, 112, 113, 113, 115, 115, 117, 118, 118, 120, 120, 122, 123, 124, 125, 125, 127};
//byte arraySCALEdorian[] =     {0, 2, 3, 5, 7, 9, 10, 12, 14, 15, 17, 19, 21, 22, 24, 26, 27, 29, 31, 33, 34, 36, 38, 39, 41, 43, 45, 46, 48, 50, 51, 53, 55, 57, 58, 60, 62, 63, 65, 67, 69, 70, 72, 74, 75, 77, 79, 81, 82, 84, 86, 87, 89, 91, 93, 94, 96, 98, 99, 101, 103, 105, 106, 108};

int  PULSEcount = 0; // counts the PULSES for each STEP so the STEP can advance to the next STEP

byte GATEOUTvalue = 0; // HIGH or LOW, direct output value of the GATE
byte GATEOUTvaluePREVIOUS = 0;
int  GATEcount = 0; // counts each clock tick between STEPs
long GATEcountTOTAL = 0; // Total of the GATEcount for each STEP
int  GATEwidthPERCENT = 50; // By default the GATE width for each PULSE of each STEP
int  GATEwidthPERCENTprevious = 0;

long GATEwidth = 0; // for GATEwidth calculation
long GATEwidthDIV24 = 0; // dividing GATEwidth by 24 for the rarchet function

long GLIDEcount = 0; // used for GLIDE mode
byte GLIDErange = 3; // default glide range, takes 25% of STEP time to reach the note, 2=50%, 3=75%, 4=100%
byte GLIDErangePREVIOUS = 0;
long GLIDErangeCOUNT = 0; // used to calculate the GATEcount for each GLIDE range

int  GLIDEvalue = 0; // small steps we increment or decrement the CV voltage between STEPs when in GLIDE MODE
int  GLIDEvalueABS = 0; // small steps we increment or decrement the CV voltage between STEPs when in GLIDE MODE in ABSOLUTE
int  MIDInoteGLIDE = 0; // MIDI note value when in GLide Mode
int  MIDInoteGLIDEprevious = 0;
byte onceFLAG = 0; // used for GLIDE calculation only once per STEP

int  PitchBEND = 0;
int  PitchBENDvalue = 0;

byte MENUdisplay = 1; // there are 4 displays in the configuration MENU
byte MENUdisplayPREVIOUS = 0;

int  ratchet2 = 0; // 2 GATES for each PULSE
int  ratchet3 = 0; // 3 GATES for each PULSE
int  ratchet4 = 0; // 4 GATES for each PULSE

int  TEMPOvalue = 60;  // used for testing the BPM UClock Library
int  TEMPOvaluePREVIOUS = 0;

int  count = 0; // counts from 0 to 7, incrementing by 1 on each loop

byte ONtime = 0; // on each clock tick this value changes from 1 to 0 to 1...
byte previousONtime = 0;

byte STEP = 0; // STEPS from 0 to 7, sequencer STEPS
byte previousSTEP = 0;
byte nextSTEP = 0;
byte STEPskippedFLAG = 0;

byte FLAG1 = 0; // used for transfering arrayPOTS values to arrayMIDInotes

byte x = 0;
byte y = 0;
byte z = 0;
byte z1 = 0;
byte z2 = 0;

byte arrayEEPROMmemory0[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};

byte arrayEEPROMmemory1[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};
byte arrayEEPROMmemory2[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};
byte arrayEEPROMmemory3[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};
byte arrayEEPROMmemory4[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};
byte arrayEEPROMmemory5[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};
byte arrayEEPROMmemory6[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};
byte arrayEEPROMmemory7[] = {
  0, 0, 0, 0, 0, 0, 0, 0, // MIDI values for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // PULSE COUNT for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // GATE MODE for all sequence
  0, 0, 0, 0, 0, 0, 0, 0, // SKIP/GLIDE/RATCHET for all sequence
  0, // TEMPO
  0, // GATE WIDHT
  0, // SCALE
  0, // KEY SHIFT
  0, // MIDI ROOT
  0  // MIDI CHANNEL
};

byte gateMODE0[] = { // GATE MODE: MUTE
  B11111,
  B10001,
  B10001,
  B10001,
  B10001,
  B10001,
  B10001,
  B11111,
};
byte gateMODE1[] = { // GATE MODE: 1 GATE FOR FIRST PULSE COUNT ONLY
  B11111,
  B10001,
  B10001,
  B10001,
  B10001,
  B11111,
  B11111,
  B11111,
};
byte gateMODE2[] = { // GATE MODE: 1 GATE FOR EACH PULSE COUNT
  B11111,
  B10001,
  B11111,
  B10001,
  B11111,
  B10001,
  B11111,
  B11111,
};
byte gateMODE3[] = { // GATE MODE: 1 GATE FOR ALL PULSE COUNTS
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
};
byte gateMODE4[] = { // ->
  B01000,
  B01100,
  B11010,
  B10001,
  B11010,
  B01100,
  B01000,
  B00000,
};
byte gateMODE5[] = { // RATCHET 2
  B01010,
  B00000,
  B00000,
  B01110,
  B00010,
  B01110,
  B01000,
  B01110,
};
byte gateMODE6[] = { // RATCHET 3
  B10101,
  B00000,
  B00000,
  B01110,
  B00010,
  B01110,
  B00010,
  B01110,
};
byte gateMODE7[] = { // RATCHET 4
  B10100,
  B01010,
  B00000,
  B01010,
  B01010,
  B01110,
  B00010,
  B00010,
};

// BPM formula; 1000/(BPM/60) = 1/4 NOTE LENGHT
// 20 BPM = 3000ms = 3sec per WHOLE 8 STEP SEQUENCE
// 320 BPM = 187.5ms = 0.1875sec per WHOLE 8 STEP SEQUENCE
// 60 BPM = 1000ms = 1sec per WHOLE 8 STEP SEQUENCE

// 20 - 320 BPM               // 1 BPM = 60000ms
// 60000ms / 20BPM = 3000ms   // 3000ms / 8 = 375ms
// 60000ms / 320BPM = 188ms   // 188ms  / 8 = 24ms


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup() {   //////////////////////////////////////////////////////////////////////// SETUP ///////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  uClock.init();  // Inits the clock
  uClock.setClock32PPQNOutput(ClockOut32PPQN); // Set the callback function for the clock output to send MIDI Sync message.
  uClock.setOnClockStartOutput(onClockStart);  // Set the callback function for MIDI Start and Stop messages.
  uClock.setOnClockStopOutput(onClockStop);  // Set something on clock start
  uClock.setTempo(60);// Set the clock BPM to 60 BPM
  uClock.start();// Starts the clock, tick-tac-tick-tac...

  //set pin(s) to input and output
  pinMode(SS_PIN, OUTPUT); // SS pin for MCP4921 DAC
  SPI.begin(); // wake up the SPI bus.
  SPI.setBitOrder(MSBFIRST);

  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  lcd.print("hello, world!");
  //  lcd.display();
  //  lcd.noDisplay();
  lcd.clear();

  lcd.createChar(0, gateMODE0);
  lcd.createChar(1, gateMODE1);
  lcd.createChar(2, gateMODE2);
  lcd.createChar(3, gateMODE3);
  lcd.createChar(4, gateMODE4);
  lcd.createChar(5, gateMODE5);
  lcd.createChar(6, gateMODE6);
  lcd.createChar(7, gateMODE7);

  // set up pins as output
  pinMode(GATEOUT, OUTPUT); // Gate Output pin

  pinMode(7, OUTPUT);
  pinMode(8, OUTPUT);
  pinMode(9, OUTPUT);

  pinMode(inputPinDP3T,  INPUT );
  pinMode(outputPinLEDS, OUTPUT);
  pinMode(inputPinPOTS,  INPUT );
  pinMode(inputPinTACTS, INPUT );

  for ( byte i = 0; i < 8; i++ ) {
    arrayDISPLAY1[i] = arrayPULSEcount[i];
    arrayDISPLAY2[i] = arrayGATEmode[i];
  }

  for ( byte i = 0; i < 8; i ++ ) {  // loads pots analog values as midi notes once at the beginning
    digitalWrite(channel[0], arrayChanA[i]);
    digitalWrite(channel[1], arrayChanB[i]);
    digitalWrite(channel[2], arrayChanC[i]);
    arrayPOTS[i]  = analogRead(inputPinPOTS);
    arrayPOTS[i]  = arrayPOTS[i] >> 3;
    arrayPOTS[i]  = 127 - arrayPOTS[i];
    arrayPOTS[i]  = map(arrayPOTS[i], 0, 127, 21, 81);
    arrayMIDInote[i] = arrayPOTS[i];
    arrayMIDInote[i] = arraySCALEdorian[arrayMIDInote[i]];
     
  }

  /*
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory1[i] =    EEPROM.read(40 + i);
      arrayEEPROMmemory1[i + 8] =  EEPROM.read(48 + i);
      arrayEEPROMmemory1[i + 16] = EEPROM.read(56 + i);
      arrayEEPROMmemory1[i + 24] = EEPROM.read(64 + i) - 3; // -3 because number that was stored in the EEPROM had to be positive but the arraySKIPslide has negative numbers
      arrayEEPROMmemory1[32] =   EEPROM.read(72);
      arrayEEPROMmemory1[33] =   EEPROM.read(73);
      arrayMIDInote[i] =        arrayEEPROMmemory1[i];
      arrayPULSEcount[i] = arrayEEPROMmemory1[i + 8];
      arrayGATEmode[i] =   arrayEEPROMmemory1[i + 16];
      arraySKIPglide[i] =  arrayEEPROMmemory1[i + 24];
      TEMPOvalue =         arrayEEPROMmemory1[32];
      GATEwidthPERCENT = arrayEEPROMmemory1[33];
      arrayDISPLAY1[i] = arrayPULSEcount[i];
      arrayDISPLAY2[i] = arrayGATEmode[i];
      arrayDISPLAY4[i] = arraySKIPglide[i];
    }
  */

#ifdef DEBUG
  Serial.begin(38400); //Set DEBUG baud rate
#else
  Serial.begin(31250); //Set MIDI baud rate
#endif

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop() {    ////////////////////////////////// LOOP LOOP LOOP LOOP LOOP LOOP LOOP LOOP /////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  uClock.setTempo(TEMPOvalue);  // every 2 clock ticks there is 1 STEP advance. 1/4 note = 0.25sec @ 60BPM

  digitalWrite(outputPinLEDS, LOW);

  digitalWrite(channel[0], arrayChanA[count]);
  digitalWrite(channel[1], arrayChanB[count]);
  digitalWrite(channel[2], arrayChanC[count]);

  arrayDP3T[count]  = digitalRead(inputPinDP3T);   //  DEPENDS IF USING TTP229 CAPACITIVE TOUCH INPUT
  arrayTACTS[count] = digitalRead(inputPinTACTS);  //  DEPENDS IF USING TTP229 CAPACITIVE TOUCH INPUT

  arrayPOTSprevious[count] = arrayPOTS[count];
  arrayPOTS[count]  = analogRead(inputPinPOTS);
  arrayPOTS[count]  = arrayPOTS[count] >> 3;
  arrayPOTS[count]  = 127 - arrayPOTS[count];
  arrayPOTS[count]  = map(arrayPOTS[count], 0, 127, 21, 81);
  arrayPOTS[count]  = arraySCALEdorian[arrayPOTS[count]];

  GATEOUTvaluePREVIOUS = GATEOUTvalue;
  digitalWrite( GATEOUT, GATEOUTvalue ); // GATE OUT

  arrayDISPLAY3[STEP] = GATEOUTvalue;
  digitalWrite( outputPinLEDS, arrayDISPLAY3[count] ); // LEDS OUT

  display1();//get data from switch combinations & display data on LCD


  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  if ( ONtime == 1 && previousONtime == 0 ) {   /////////////////// onSTEP = 1  ///////////// this is the beginning of the a STEP /////////////////////////////
    GATEcountTOTAL = GATEcount;
    GATEcount = 0; // resets the GATEcount at the beginning of each step
    PULSEcount = PULSEcount + 1; // counting the PULSEs for each STEP
    GATEwidth = ( GATEcountTOTAL * GATEwidthPERCENT ) / 100; // calculates GATEwidth in counts instead of percents
    GATEwidthDIV24 = GATEcountTOTAL / 24; // for the ratchet calculations
    ratchet2 = 6 * GATEwidthDIV24; // 2 GATES per PULSE
    ratchet3 = 4 * GATEwidthDIV24; // 3 GATES per PULSE
    ratchet4 = 3 * GATEwidthDIV24; // 4 GATES per PULSE
    onceFLAG = 1; // used for GLIDE calculation only once per STEP
    previousONtime = 1;
  }

  if ( ONtime == 0 && previousONtime == 1 ) {   /////////////////// onSTEP = 0  ///////////// this is the middle of the a STEP ////////////////////////////////
    previousONtime = 0;
  }

  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////////// GATE MODES begin /////////////////////////////////////////////////////////////////////////////////////


  if ( arrayGATEmode[STEP] == 0 ) {  ////////////////////////////////////////////////////////////// GATE MODE 0 ////////////////////////////////////////////////
    GATEOUTvalue = LOW;
  }

  if ( arrayGATEmode[STEP] == 1 && GATEcount != 0 ) {  //////////////////////////////////////////// GATE MODE 1 ////////////////////////////////////////////////
    if ( GATEcount <= GATEwidth && PULSEcount == 1 ) {
      GATEOUTvalue = HIGH;
    }
    else {
      GATEOUTvalue = LOW;
    }
  }

  if ( arrayGATEmode[STEP] == 2  && GATEcount != 0 && arrayPULSEcount[STEP] >= PULSEcount ) {  //// GATE MODE 2 ////////////////////////////////////////////////
    if ( GATEcount <= GATEwidth && PULSEcount >= 1 ) {
      GATEOUTvalue = HIGH;
    }
    else {
      GATEOUTvalue = LOW;
    }
  }

  if ( arrayGATEmode[STEP] == 3 && arrayPULSEcount[STEP] == 1 ) { ///////////////////////////////// GATE MODE 3 ////////////////////////////////////////////////
    if ( GATEcount <= ( GATEcountTOTAL - ratchet4 ) ) {
      GATEOUTvalue = HIGH;
    }
    else {
      GATEOUTvalue = LOW;
    }
  }

  if ( arrayGATEmode[STEP] == 3 && arrayPULSEcount[STEP] > 1 ) { ///////////////////////////////// GATE MODE 3 /////////////////////////////////////////////////
    if ( PULSEcount >= 1 && PULSEcount < arrayPULSEcount[STEP] ) {
      GATEOUTvalue = HIGH;
    }
    else if ( PULSEcount == arrayPULSEcount[STEP] && ( GATEcount <= ( GATEcountTOTAL - ratchet4 ) ) ) {
      GATEOUTvalue = HIGH;
    }
    else {
      GATEOUTvalue = LOW;
    }
  }

  /////////////////////////////////////////////////// GATE MODES end ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////// SKIP/GLIDE/RATCHET MODES begin ///////////////////////////////////////////////////////////////////////////////////////////////////////


  if ( arraySKIPglide[STEP] == -1 || arraySKIPglide[STEP] == 3 ) {  // RATCHET 2 (GATE/2) ///////////////////////////////////////////////////////////////////////

    if ( arrayGATEmode[STEP] == 1 && PULSEcount == 1 ) {

      if ( GATEcount >= 0 && GATEcount < ratchet2 ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= ratchet2  && GATEcount < (2 * ratchet2) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (2 * ratchet2) && GATEcount < (3 * ratchet2) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (3 * ratchet2) && GATEcount < (4 * ratchet2) ) {
        GATEOUTvalue = LOW;
      }
    }

    if ( arrayGATEmode[STEP] == 2 ) {

      if ( GATEcount >= 0 && GATEcount < ratchet2 ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= ratchet2 && GATEcount < (2 * ratchet2) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (2 * ratchet2) && GATEcount < (3 * ratchet2) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (3 * ratchet2) && GATEcount < (4 * ratchet2) ) {
        GATEOUTvalue = LOW;
      }
    }

  } /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  if ( arraySKIPglide[STEP] == -2 || arraySKIPglide[STEP] == 4 ) {  // RATCHET 3 (GATE/3) ///////////////////////////////////////////////////////////////////////

    if ( arrayGATEmode[STEP] == 1 && PULSEcount == 1 ) {

      if ( GATEcount >= 0  && GATEcount < ratchet3 ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= ratchet3  && GATEcount < (2 * ratchet3) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (2 * ratchet3) && GATEcount < (3 * ratchet3) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (3 * ratchet3) && GATEcount < (4 * ratchet3) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (4 * ratchet3) && GATEcount < (5 * ratchet3) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (5 * ratchet3) && GATEcount < (6 * ratchet3) ) {
        GATEOUTvalue = LOW;
      }
    }

    if ( arrayGATEmode[STEP] == 2 ) {

      if ( GATEcount >= 0  && GATEcount < ratchet3 ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= ratchet3  && GATEcount < (2 * ratchet3) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (2 * ratchet3) && GATEcount < (3 * ratchet3) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (3 * ratchet3) && GATEcount < (4 * ratchet3) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (4 * ratchet3) && GATEcount < (5 * ratchet3) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (5 * ratchet3) && GATEcount < (6 * ratchet3) ) {
        GATEOUTvalue = LOW;
      }
    }
  } //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  if ( arraySKIPglide[STEP] == -3 || arraySKIPglide[STEP] == 5 ) {  // RATCHET 4 (GATE/4) ////////////////////////////////////////////////////////////////////////////////

    if ( arrayGATEmode[STEP] == 1 && PULSEcount == 1 ) {

      if ( GATEcount >= 0  && GATEcount < ratchet4 ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= ratchet4  && GATEcount < (2 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (2 * ratchet4) && GATEcount < (3 * ratchet4) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (3 * ratchet4) && GATEcount < (4 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (4 * ratchet4) && GATEcount < (5 * ratchet4) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (5 * ratchet4) && GATEcount < (6 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (6 * ratchet4) && GATEcount < (7 * ratchet4) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (7 * ratchet4) && GATEcount < (8 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
    }
    if ( arrayGATEmode[STEP] == 2 ) {

      if ( GATEcount >= 0  && GATEcount < ratchet4 ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= ratchet4  && GATEcount < (2 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (2 * ratchet4) && GATEcount < (3 * ratchet4) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (3 * ratchet4) && GATEcount < (4 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (4 * ratchet4) && GATEcount < (5 * ratchet4) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (5 * ratchet4) && GATEcount < (6 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
      if ( GATEcount >= (6 * ratchet4) && GATEcount < (7 * ratchet4) ) {
        GATEOUTvalue = HIGH;
      }
      if ( GATEcount >= (7 * ratchet4) && GATEcount < (8 * ratchet4) ) {
        GATEOUTvalue = LOW;
      }
    }
  }

  //////////////////////// SKIP/GLIDE/RATCHET MODES end ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  ///////////////////////// NO GLIDE MODE begins ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  if ( arraySKIPglide[STEP] <= 0 && PULSEcount == 1 && onceFLAG == 1 ) {

    outputValue = arrayCVvalues[arrayMIDInote[STEP] - 21];

    if ( arrayMIDInote[STEP] < 21 ) {
      outputValue = arrayCVvalues[0];
    }

    if ( arrayMIDInote[STEP] > 81 ) {
      outputValue = arrayCVvalues[60];
    }

    PORTB &= ~(1 << 2); // set bit 2 in PORTB to 0  //digitalWrite(10, LOW);
    data = highByte(outputValue);
    data = 0b00001111 & data;
    data = 0b00110000 | data; // 0b00010000 = (GAIN 2X)   // 0b00110000 = (GAIN 1X)
    SPI.transfer(data);
    data = lowByte(outputValue);
    SPI.transfer(data);
    PORTB |= (1 << 2); // set bit 2 in PORTB to 1  //digitalWrite(10, HIGH);

  }

  //////////////////////// NO GLIDE MODE ends ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  //////////////////////// GLIDE MODE begins /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  if ( ( arraySKIPglide[STEP] == 1 || arraySKIPglide[STEP] >= 3 ) && PULSEcount == 1 && onceFLAG == 1 ) { // CV array range limited between MIDI note 21 (0V) and 81 (5V)

    MIDInoteGLIDE = arrayMIDInote[previousSTEP];
    GLIDEvalue = arrayMIDInote[STEP] - arrayMIDInote[previousSTEP];

    switch (GLIDErange) {  // 1=25% 2=50% 3=75% 4=100%
      case 1:
        GLIDErangeCOUNT = GATEcountTOTAL / 4; // calculates the number of counts for 25% of the STEP
        break;
      case 2:
        GLIDErangeCOUNT = GATEcountTOTAL / 2 ; // calculates the number of counts for 50% of the STEP
        break;
      case 3:
        GLIDErangeCOUNT = (GATEcountTOTAL / 4) * 3; // calculates the number of counts for 75% of the STEP
        break;
      case 4:
        GLIDErangeCOUNT = GATEcountTOTAL; // calculates the number of counts for 100% of the STEP
        break;
      default:
        GLIDErangeCOUNT = GATEcountTOTAL / 4; // calculates the number of counts for 25% of the STEP
        break;
    }

    GLIDErangeCOUNT = GLIDErangeCOUNT / GLIDEvalue;
    GLIDEvalueABS = abs(GLIDEvalue);
    PitchBENDvalue  = map(GLIDEvalueABS, 0, 127, 0, 64);
    PitchBENDvalue = 64 / PitchBENDvalue;
    GLIDEcount = 0;
    PitchBEND = 0;
    onceFLAG = 0;

    if ( GLIDErangeCOUNT < 0 ) {
      GLIDErangeCOUNT = -1 * GLIDErangeCOUNT;
    }

  }

  MIDInoteGLIDEprevious = MIDInoteGLIDE;

  if ( ( arraySKIPglide[STEP] == 1 || arraySKIPglide[STEP] >= 3 ) ) { // && MIDInoteGLIDE >= 21 ) {

    if ( ( ( GATEcount + GATEwidthDIV24 ) < GATEcountTOTAL ) && ( GATEcount > GLIDEcount ) ) {

      if ( GLIDEvalue > 0 && MIDInoteGLIDE < arrayMIDInote[STEP] ) {

        MIDInoteGLIDE = MIDInoteGLIDE + 1;
        GLIDEcount = GLIDEcount + GLIDErangeCOUNT;
        PitchBEND = PitchBEND + PitchBENDvalue;
      }

      if ( GLIDEvalue < 0 && MIDInoteGLIDE > arrayMIDInote[STEP] ) {

        MIDInoteGLIDE = MIDInoteGLIDE - 1;
        GLIDEcount = GLIDEcount + GLIDErangeCOUNT;
        PitchBEND = PitchBEND + PitchBENDvalue;
      }
    }

    if ( MIDInoteGLIDE >= 21 && MIDInoteGLIDE <= 81 ) {
      outputValue = arrayCVvalues[MIDInoteGLIDE - 21];
    }

    if ( MIDInoteGLIDE < 21 ) {
      outputValue = arrayCVvalues[0];
    }

    if ( MIDInoteGLIDE > 81 ) {
      outputValue = arrayCVvalues[60];
    }

    PORTB &= ~(1 << 2); // set bit 2 in PORTB to 0  //digitalWrite(10, LOW);
    data = highByte(outputValue);
    data = 0b00001111 & data;
    data = 0b00110000 | data; // 0b00010000 = (GAIN 2X)   // 0b00110000 = (GAIN 1X)
    SPI.transfer(data);
    data = lowByte(outputValue);
    SPI.transfer(data);
    PORTB |= (1 << 2); // set bit 2 in PORTB to 1  //digitalWrite(10, HIGH);

  }

  //////////////////////// GLIDE MODE ends ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  //////////////////////// MIDI OUT begins ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  if ( GATEOUTvalue == HIGH && GATEOUTvaluePREVIOUS == LOW && arrayGATEmode[STEP] != 0 && arraySKIPglide[STEP] <= 0 ) {   //////// GLIDE MODE OFF /////////////////////////
    if ( arraySKIPglide[previousSTEP] > 0 ) {
      ControlChange ( 0xE0, 0x00, 64 );
    }
    noteOn(0x80, arrayMIDInote[previousSTEP], 0x00);
    if ( arrayMIDInote[STEP] != 0 ) {
      noteOn(0x90, arrayMIDInote[STEP], 0x7F);
    }
  }

  if ( GATEOUTvalue == LOW && GATEOUTvaluePREVIOUS == HIGH && arrayGATEmode[STEP] != 0 && arraySKIPglide[STEP] <= 0 ) {   //////// GLIDE MODE OFF /////////////////////////
    noteOn(0x80, arrayMIDInote[STEP], 0x00);
  }

  if ( ( MIDInoteGLIDE != MIDInoteGLIDEprevious ) && arrayGATEmode[STEP] != 0 && arraySKIPglide[STEP] >= 1 ) {   //////// GLIDE MODE ON ///////////////////////////////////

    if ( PitchBEND == PitchBENDvalue ) {
      ControlChange ( 0xE0, 0x00, 0x00 );
      noteOn(0x90, arrayMIDInote[STEP], 0x7F);
      ControlChange ( 0xE0, 0x00, PitchBENDvalue );
    }

    if ( PitchBEND > PitchBENDvalue && PitchBEND < 64 ) {
      ControlChange ( 0xE0, 0x00, PitchBEND );
    }

  }

  //////////////////////// MIDI OUT ends ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  //////////////////////// BEFORE the END of MAIN LOOP //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  if ( (PULSEcount == arrayPULSEcount[STEP] + 1 ) && GATEcount == 0 ) { // ADVANCE 1 STEP only when PULSE count is done
    STEP = STEP + 1;
    PULSEcount = 1;
  }

  if ( STEP >= 8 ) {                    // resets the STEP counter
    STEP = 0;
  }

  if ( arraySKIPglide[STEP] == 2 ) {    // SKIPS ONE STEP when SKIPPING is ACTIVE
    STEP = STEP + 1;
    STEPskippedFLAG = 1; // used to caluclate the the previousSTEP value
  }

  if ( STEP >= 8 ) {                    // resets the STEP counter
    STEP = 0;
  }

  if ( STEP == 7 ) {
    nextSTEP = 0;
  }
  else {
    nextSTEP = STEP + 1;
  }

  if ( STEP == 0 ) {
    previousSTEP = 7;
  }
  else {
    previousSTEP = STEP - 1;
  }

  if ( previousSTEP > 0 && STEPskippedFLAG == 1 ) {
    previousSTEP = previousSTEP - 1;
    STEPskippedFLAG = 0;
  }

  if ( previousSTEP == 0 && STEPskippedFLAG == 1 ) {
    previousSTEP = 7;
    STEPskippedFLAG = 0;
  }

  GATEcount++;

  count++;
  if ( count == 8 ) {
    count = 0;
  }

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// END OF MAIN LOOP /////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// The callback function wich will be called by Clock each Pulse of 96PPQN clock resolution.
void ClockOut32PPQN(uint32_t * tick) {
  ONtime = !ONtime;
}

// The callback function wich will be called when clock starts by using Clock.start() method.
void onClockStart() {
  //
}

// The callback function wich will be called when clock stops by using Clock.stop() method.
void onClockStop() {
  //
}

// plays a MIDI note. Doesn't check to see that cmd is greater than 127, or that
// data values are less than 127:
void noteOn(int cmd, int pitch, int velocity) {
#ifndef DEBUG
  Serial.write(cmd);
  Serial.write(pitch);
  Serial.write(velocity);
#endif
}

void ProgramChange(int program, int channel) {
#ifndef DEBUG
  Serial.write(program);
  Serial.write(channel);
#endif
}

void ControlChange(int cmd, int cc, int value) {
#ifndef DEBUG
  Serial.write(cmd);
  Serial.write(cc);
  Serial.write(value);
#endif
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void display1() { /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  if ( arrayPOTS[count] != arrayPOTSprevious[count] ) {
    arrayMIDInote[count] = arrayPOTS[count];
  }

  if ( arrayDP3T[5] == 1 && arrayTACTS[0] == 1 && arrayTACTSprevious[0] == 0 ) {
    arrayTACTSprevious[0] = 1;
    MENUdisplay = 1;
  }
  if ( arrayDP3T[5] == 1 && arrayTACTS[1] == 1 && arrayTACTSprevious[1] == 0 ) {
    arrayTACTSprevious[1] = 1;
    MENUdisplay = 1;
  }
  if ( arrayDP3T[5] == 1 && arrayTACTS[2] == 1 && arrayTACTSprevious[2] == 0 ) {
    arrayTACTSprevious[2] = 1;
    MENUdisplay = 2;
  }
  if ( arrayDP3T[5] == 1 && arrayTACTS[3] == 1 && arrayTACTSprevious[3] == 0 ) {
    arrayTACTSprevious[3] = 1;
    MENUdisplay = 2;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////// LOAD/SAVE MODES (3) ///////////////////////////////////////////////////////////////////////////

  if ( arrayDP3T[3] == 1 && arrayDP3Tprevious[3] == 0 ) {  // happens only once when switched into SAVE/LOAD MODE
    arrayDP3Tprevious[3] = 1;
    lcd.setCursor(0, 1);
    lcd.print("ACTIVE SEQ.SAVE>");
    lcd.setCursor(0, 0);
    lcd.print("   2-8 SEQ.LOAD>");
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory0[i] = arrayMIDInote[i];             // MIDI NOTE for all sequence
      arrayEEPROMmemory0[i + 8] = arrayPULSEcount[i];       // PULSE COUNT for all sequence
      arrayEEPROMmemory0[i + 16] = arrayGATEmode[i];        // GATE MODE for all sequence
      arrayEEPROMmemory0[i + 24] = (arraySKIPglide[i]) + 3; // SKIP/GLIDE/RATCHET for all sequence  // +3 to eliminate the negative number writing to EEPROM, when reading EEPROM need to subtract
      arrayEEPROMmemory0[32] = TEMPOvalue;                  // TEMPO
      arrayEEPROMmemory0[33] = GATEwidthPERCENT;            // GATE WIDHT
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[0] == 1 && arrayTACTSprevious[0] == 0 ) { //SHOW LOC 0
    arrayTACTSprevious[0] = 1;
    FLAG1 = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        arrayEEPROMmemory0[i] = arrayMIDInote[i];
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        arrayEEPROMmemory0[i] = arrayMIDInote[i];
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[1] == 1 && arrayTACTSprevious[1] == 0 ) { //SHOW LOC 1
    arrayTACTSprevious[1] = 1;
    FLAG1 = 0;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory1[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory1[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[2] == 1 && arrayTACTSprevious[2] == 0 ) { //SHOW LOC 2
    arrayTACTSprevious[2] = 1;
    FLAG1 = 0;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory2[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory2[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[3] == 1 && arrayTACTSprevious[3] == 0 ) { //SHOW LOC 3
    arrayTACTSprevious[3] = 1;
    FLAG1 = 0;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory3[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory3[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[4] == 1 && arrayTACTSprevious[4] == 0 ) { //SHOW LOC 4
    arrayTACTSprevious[4] = 1;
    FLAG1 = 0;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory4[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory4[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[5] == 1 && arrayTACTSprevious[5] == 0 ) { //SHOW LOC 5
    arrayTACTSprevious[5] = 1;
    FLAG1 = 0;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory5[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory5[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[6] == 1 && arrayTACTSprevious[6] == 0 ) { //SHOW LOC 6
    arrayTACTSprevious[6] = 1;
    FLAG1 = 0;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory6[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory6[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[7] == 1 && arrayTACTSprevious[7] == 0 ) { //SHOW LOC 7
    arrayTACTSprevious[7] = 1;
    FLAG1 = 0;
    for ( byte i = 0; i < 8; i ++ ) {
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory7[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory7[i]);
      }
    }
  }

  /////////////////////////////////////////////////////////////////////////////////////////// TESTING BEGIN

#ifdef DEBUG
  if ( arrayPOTS[7] == 81 ) {

    x = arrayDP3T[3];
    y = arrayTACTS[1];
    z = arrayTACTSprevious[1];
    z1 = arrayDP3T[6];
    z2 = arrayDP3Tprevious[6];

    Serial.print(" arrayDP3T[3]= ");
    Serial.print(x);
    Serial.print(" arrayTACTS[1]= ");
    Serial.print(y);
    Serial.print(" arrayTACTSprevious[1]= ");
    Serial.print(z);
    Serial.print(" arrayDP3T[6]= ");
    Serial.print(z1);
    Serial.print(" arrayDP3Tprevious[6]= ");
    Serial.println(z2);
  }
#endif

  /////////////////////////////////////////////////////////////////////////////////////////// TESTING END


  if ( arrayDP3T[3] == 1 && arrayTACTS[1] == 1 && arrayDP3T[6] == 1 && arrayDP3Tprevious[6] == 0  ) {  // 2-8 SEQUENCE LOAD FROM EEPROM /////////////////////////////////////////////////////////
    arrayDP3Tprevious[6] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory1[i] =      EEPROM.read(40 + i);
      arrayEEPROMmemory1[i + 8] =  EEPROM.read(48 + i);
      arrayEEPROMmemory1[i + 16] = EEPROM.read(56 + i);
      arrayEEPROMmemory1[i + 24] = EEPROM.read(64 + i) - 3; // -3 because number that was stored in the EEPROM had to be positive but the arraySKIPslide has negative numbers
      arrayEEPROMmemory1[32] =     EEPROM.read(72);
      arrayEEPROMmemory1[33] =     EEPROM.read(73);
      arrayMIDInote[i] =    arrayEEPROMmemory1[i];
      arrayPULSEcount[i] =  arrayEEPROMmemory1[i + 8];
      arrayGATEmode[i] =    arrayEEPROMmemory1[i + 16];
      arraySKIPglide[i] =   arrayEEPROMmemory1[i + 24];
      TEMPOvalue =          arrayEEPROMmemory1[32];
      GATEwidthPERCENT =    arrayEEPROMmemory1[33];
      arrayDISPLAY1[i] =    arrayPULSEcount[i];
      arrayDISPLAY2[i] =    arrayGATEmode[i];
      arrayDISPLAY4[i] =    arraySKIPglide[i];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory1[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory1[i]);
      }      
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[2] == 1 && arrayDP3T[6] == 1 && arrayDP3Tprevious[6] == 0  ) {  // 2-8 SEQUENCE LOAD FROM EEPROM /////////////////////////////////////////////////////////
    arrayDP3Tprevious[6] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory2[i] =      EEPROM.read(80 + i);
      arrayEEPROMmemory2[i + 8] =  EEPROM.read(88 + i);
      arrayEEPROMmemory2[i + 16] = EEPROM.read(96 + i);
      arrayEEPROMmemory2[i + 24] = EEPROM.read(104 + i) - 3;
      arrayEEPROMmemory2[32] =     EEPROM.read(112);
      arrayEEPROMmemory2[33] =     EEPROM.read(113);
      arrayMIDInote[i] =    arrayEEPROMmemory2[i];
      arrayPULSEcount[i] =  arrayEEPROMmemory2[i + 8];
      arrayGATEmode[i] =    arrayEEPROMmemory2[i + 16];
      arraySKIPglide[i] =   arrayEEPROMmemory2[i + 24];
      TEMPOvalue =          arrayEEPROMmemory2[32];
      GATEwidthPERCENT =    arrayEEPROMmemory2[33];
      arrayDISPLAY1[i] =    arrayPULSEcount[i];
      arrayDISPLAY2[i] =    arrayGATEmode[i];
      arrayDISPLAY4[i] =    arraySKIPglide[i];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory2[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory2[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[3] == 1 && arrayDP3T[6] == 1 && arrayDP3Tprevious[6] == 0  ) {  // 2-8 SEQUENCE LOAD FROM EEPROM /////////////////////////////////////////////////////////
    arrayDP3Tprevious[6] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory3[i] =      EEPROM.read(120 + i);
      arrayEEPROMmemory3[i + 8] =  EEPROM.read(128 + i);
      arrayEEPROMmemory3[i + 16] = EEPROM.read(136 + i);
      arrayEEPROMmemory3[i + 24] = EEPROM.read(144 + i) - 3; // 
      arrayEEPROMmemory3[32] =     EEPROM.read(152);
      arrayEEPROMmemory3[33] =     EEPROM.read(153);
      arrayMIDInote[i] =    arrayEEPROMmemory3[i];
      arrayPULSEcount[i] =  arrayEEPROMmemory3[i + 8];
      arrayGATEmode[i] =    arrayEEPROMmemory3[i + 16];
      arraySKIPglide[i] =   arrayEEPROMmemory3[i + 24];
      TEMPOvalue =          arrayEEPROMmemory3[32];
      GATEwidthPERCENT =    arrayEEPROMmemory3[33];
      arrayDISPLAY1[i] =    arrayPULSEcount[i];
      arrayDISPLAY2[i] =    arrayGATEmode[i];
      arrayDISPLAY4[i] =    arraySKIPglide[i];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory3[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory3[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[4] == 1 && arrayDP3T[6] == 1 && arrayDP3Tprevious[6] == 0  ) {  // 2-8 SEQUENCE LOAD FROM EEPROM /////////////////////////////////////////////////////////
    arrayDP3Tprevious[6] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory4[i] =      EEPROM.read(160 + i);
      arrayEEPROMmemory4[i + 8] =  EEPROM.read(168 + i);
      arrayEEPROMmemory4[i + 16] = EEPROM.read(176 + i);
      arrayEEPROMmemory4[i + 24] = EEPROM.read(184 + i) - 3; // 
      arrayEEPROMmemory4[32] =     EEPROM.read(192);
      arrayEEPROMmemory4[33] =     EEPROM.read(193);
      arrayMIDInote[i] =    arrayEEPROMmemory4[i];
      arrayPULSEcount[i] =  arrayEEPROMmemory4[i + 8];
      arrayGATEmode[i] =    arrayEEPROMmemory4[i + 16];
      arraySKIPglide[i] =   arrayEEPROMmemory4[i + 24];
      TEMPOvalue =          arrayEEPROMmemory4[32];
      GATEwidthPERCENT =    arrayEEPROMmemory4[33];
      arrayDISPLAY1[i] =    arrayPULSEcount[i];
      arrayDISPLAY2[i] =    arrayGATEmode[i];
      arrayDISPLAY4[i] =    arraySKIPglide[i];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory4[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory4[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[5] == 1 && arrayDP3T[6] == 1 && arrayDP3Tprevious[6] == 0  ) {  // 2-8 SEQUENCE LOAD FROM EEPROM /////////////////////////////////////////////////////////
    arrayDP3Tprevious[6] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory5[i] =      EEPROM.read(200 + i);
      arrayEEPROMmemory5[i + 8] =  EEPROM.read(208 + i);
      arrayEEPROMmemory5[i + 16] = EEPROM.read(216 + i);
      arrayEEPROMmemory5[i + 24] = EEPROM.read(224 + i) - 3; // 
      arrayEEPROMmemory5[32] =     EEPROM.read(232);
      arrayEEPROMmemory5[33] =     EEPROM.read(233);
      arrayMIDInote[i] =    arrayEEPROMmemory5[i];
      arrayPULSEcount[i] =  arrayEEPROMmemory5[i + 8];
      arrayGATEmode[i] =    arrayEEPROMmemory5[i + 16];
      arraySKIPglide[i] =   arrayEEPROMmemory5[i + 24];
      TEMPOvalue =          arrayEEPROMmemory5[32];
      GATEwidthPERCENT =    arrayEEPROMmemory5[33];
      arrayDISPLAY1[i] =    arrayPULSEcount[i];
      arrayDISPLAY2[i] =    arrayGATEmode[i];
      arrayDISPLAY4[i] =    arraySKIPglide[i];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory5[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory5[i]);
      }      
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[6] == 1 && arrayDP3T[6] == 1 && arrayDP3Tprevious[6] == 0  ) {  // 2-8 SEQUENCE LOAD FROM EEPROM /////////////////////////////////////////////////////////
    arrayDP3Tprevious[6] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory6[i] =      EEPROM.read(240 + i);
      arrayEEPROMmemory6[i + 8] =  EEPROM.read(248 + i);
      arrayEEPROMmemory6[i + 16] = EEPROM.read(256 + i);
      arrayEEPROMmemory6[i + 24] = EEPROM.read(264 + i) - 3; // 
      arrayEEPROMmemory6[32] =     EEPROM.read(272);
      arrayEEPROMmemory6[33] =     EEPROM.read(273);
      arrayMIDInote[i] =    arrayEEPROMmemory6[i];
      arrayPULSEcount[i] =  arrayEEPROMmemory6[i + 8];
      arrayGATEmode[i] =    arrayEEPROMmemory6[i + 16];
      arraySKIPglide[i] =   arrayEEPROMmemory6[i + 24];
      TEMPOvalue =          arrayEEPROMmemory6[32];
      GATEwidthPERCENT =    arrayEEPROMmemory6[33];
      arrayDISPLAY1[i] =    arrayPULSEcount[i];
      arrayDISPLAY2[i] =    arrayGATEmode[i];
      arrayDISPLAY4[i] =    arraySKIPglide[i];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory6[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory6[i]);
      }
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[7] == 1 && arrayDP3T[6] == 1 && arrayDP3Tprevious[6] == 0  ) {  // 2-8 SEQUENCE LOAD FROM EEPROM /////////////////////////////////////////////////////////
    arrayDP3Tprevious[6] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      arrayEEPROMmemory7[i] =      EEPROM.read(280 + i);
      arrayEEPROMmemory7[i + 8] =  EEPROM.read(288 + i);
      arrayEEPROMmemory7[i + 16] = EEPROM.read(296 + i);
      arrayEEPROMmemory7[i + 24] = EEPROM.read(304 + i) - 3; // 
      arrayEEPROMmemory7[32] =     EEPROM.read(312);
      arrayEEPROMmemory7[33] =     EEPROM.read(313);
      arrayMIDInote[i] =    arrayEEPROMmemory7[i];
      arrayPULSEcount[i] =  arrayEEPROMmemory7[i + 8];
      arrayGATEmode[i] =    arrayEEPROMmemory7[i + 16];
      arraySKIPglide[i] =   arrayEEPROMmemory7[i + 24];
      TEMPOvalue =          arrayEEPROMmemory7[32];
      GATEwidthPERCENT =    arrayEEPROMmemory7[33];
      arrayDISPLAY1[i] =    arrayPULSEcount[i];
      arrayDISPLAY2[i] =    arrayGATEmode[i];
      arrayDISPLAY4[i] =    arraySKIPglide[i];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory7[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory7[i]);
      }
    }
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////////////


  if ( arrayDP3T[3] == 1 && arrayTACTS[1] == 1 && arrayDP3T[7] == 1 && arrayDP3Tprevious[7] == 0  ) {  // ACTIVE SEQUENCE SAVE TO EEPROM //////////////////////////
    arrayDP3Tprevious[7] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      EEPROM.write(40 + i, arrayEEPROMmemory0[i]);   // 40 is the first address of arrayEEPROMmemory1
      EEPROM.write(48 + i, arrayEEPROMmemory0[i + 8]);
      EEPROM.write(56 + i, arrayEEPROMmemory0[i + 16]);
      EEPROM.write(64 + i, arrayEEPROMmemory0[i + 24]);
      EEPROM.write(72,     arrayEEPROMmemory0[32]);
      EEPROM.write(73,     arrayEEPROMmemory0[33]);
      arrayEEPROMmemory1[i] =    arrayEEPROMmemory0[i];
      arrayEEPROMmemory1[i + 8] =  arrayEEPROMmemory0[i + 8];
      arrayEEPROMmemory1[i + 16] = arrayEEPROMmemory0[i + 16];
      arrayEEPROMmemory1[i + 24] = arrayEEPROMmemory0[i + 24];
      arrayEEPROMmemory1[32] =   arrayEEPROMmemory0[32];
      arrayEEPROMmemory1[33] =   arrayEEPROMmemory0[33];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }  
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[2] == 1 && arrayDP3T[7] == 1 && arrayDP3Tprevious[7] == 0  ) {  // ACTIVE SEQUENCE SAVE TO EEPROM //////////////////////////
    arrayDP3Tprevious[7] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      EEPROM.write(80 + i, arrayEEPROMmemory0[i]);   // 80 is the first address of arrayEEPROMmemory2
      EEPROM.write(88 + i, arrayEEPROMmemory0[i + 8]);
      EEPROM.write(96 + i, arrayEEPROMmemory0[i + 16]);
      EEPROM.write(104 + i, arrayEEPROMmemory0[i + 24]);
      EEPROM.write(112,     arrayEEPROMmemory0[32]);
      EEPROM.write(113,     arrayEEPROMmemory0[33]);
      arrayEEPROMmemory2[i] = arrayEEPROMmemory0[i];
      arrayEEPROMmemory2[i + 8] =  arrayEEPROMmemory0[i + 8];
      arrayEEPROMmemory2[i + 16] = arrayEEPROMmemory0[i + 16];
      arrayEEPROMmemory2[i + 24] = arrayEEPROMmemory0[i + 24];
      arrayEEPROMmemory2[32] =   arrayEEPROMmemory0[32];
      arrayEEPROMmemory2[33] =   arrayEEPROMmemory0[33];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }  
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[3] == 1 && arrayDP3T[7] == 1 && arrayDP3Tprevious[7] == 0  ) {  // ACTIVE SEQUENCE SAVE TO EEPROM //////////////////////////
    arrayDP3Tprevious[7] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      EEPROM.write(120 + i, arrayEEPROMmemory0[i]);   // 120 is the first address of arrayEEPROMmemory3
      EEPROM.write(128 + i, arrayEEPROMmemory0[i + 8]);
      EEPROM.write(136 + i, arrayEEPROMmemory0[i + 16]);
      EEPROM.write(144 + i, arrayEEPROMmemory0[i + 24]);
      EEPROM.write(152,     arrayEEPROMmemory0[32]);
      EEPROM.write(153,     arrayEEPROMmemory0[33]);
      arrayEEPROMmemory3[i] = arrayEEPROMmemory0[i];
      arrayEEPROMmemory3[i + 8] =  arrayEEPROMmemory0[i + 8];
      arrayEEPROMmemory3[i + 16] = arrayEEPROMmemory0[i + 16];
      arrayEEPROMmemory3[i + 24] = arrayEEPROMmemory0[i + 24];
      arrayEEPROMmemory3[32] =   arrayEEPROMmemory0[32];
      arrayEEPROMmemory3[33] =   arrayEEPROMmemory0[33];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }  
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[4] == 1 && arrayDP3T[7] == 1 && arrayDP3Tprevious[7] == 0  ) {  // ACTIVE SEQUENCE SAVE TO EEPROM //////////////////////////
    arrayDP3Tprevious[7] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      EEPROM.write(160 + i, arrayEEPROMmemory0[i]);   // 160 is the first address of arrayEEPROMmemory4
      EEPROM.write(168 + i, arrayEEPROMmemory0[i + 8]);
      EEPROM.write(176 + i, arrayEEPROMmemory0[i + 16]);
      EEPROM.write(184 + i, arrayEEPROMmemory0[i + 24]);
      EEPROM.write(192,     arrayEEPROMmemory0[32]);
      EEPROM.write(193,     arrayEEPROMmemory0[33]);
      arrayEEPROMmemory4[i] = arrayEEPROMmemory0[i];
      arrayEEPROMmemory4[i + 8] =  arrayEEPROMmemory0[i + 8];
      arrayEEPROMmemory4[i + 16] = arrayEEPROMmemory0[i + 16];
      arrayEEPROMmemory4[i + 24] = arrayEEPROMmemory0[i + 24];
      arrayEEPROMmemory4[32] =   arrayEEPROMmemory0[32];
      arrayEEPROMmemory4[33] =   arrayEEPROMmemory0[33];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }  
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[5] == 1 && arrayDP3T[7] == 1 && arrayDP3Tprevious[7] == 0  ) {  // ACTIVE SEQUENCE SAVE TO EEPROM //////////////////////////
    arrayDP3Tprevious[7] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      EEPROM.write(200 + i, arrayEEPROMmemory0[i]);   // 200 is the first address of arrayEEPROMmemory5
      EEPROM.write(208 + i, arrayEEPROMmemory0[i + 8]);
      EEPROM.write(216 + i, arrayEEPROMmemory0[i + 16]);
      EEPROM.write(224 + i, arrayEEPROMmemory0[i + 24]);
      EEPROM.write(232,     arrayEEPROMmemory0[32]);
      EEPROM.write(233,     arrayEEPROMmemory0[33]);
      arrayEEPROMmemory5[i] = arrayEEPROMmemory0[i];
      arrayEEPROMmemory5[i + 8] =  arrayEEPROMmemory0[i + 8];
      arrayEEPROMmemory5[i + 16] = arrayEEPROMmemory0[i + 16];
      arrayEEPROMmemory5[i + 24] = arrayEEPROMmemory0[i + 24];
      arrayEEPROMmemory5[32] =   arrayEEPROMmemory0[32];
      arrayEEPROMmemory5[33] =   arrayEEPROMmemory0[33];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }  
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[6] == 1 && arrayDP3T[7] == 1 && arrayDP3Tprevious[7] == 0  ) {  // ACTIVE SEQUENCE SAVE TO EEPROM //////////////////////////
    arrayDP3Tprevious[7] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      EEPROM.write(240 + i, arrayEEPROMmemory0[i]);   // 240 is the first address of arrayEEPROMmemory6
      EEPROM.write(248 + i, arrayEEPROMmemory0[i + 8]);
      EEPROM.write(256 + i, arrayEEPROMmemory0[i + 16]);
      EEPROM.write(264 + i, arrayEEPROMmemory0[i + 24]);
      EEPROM.write(272,     arrayEEPROMmemory0[32]);
      EEPROM.write(273,     arrayEEPROMmemory0[33]);
      arrayEEPROMmemory6[i] = arrayEEPROMmemory0[i];
      arrayEEPROMmemory6[i + 8] =  arrayEEPROMmemory0[i + 8];
      arrayEEPROMmemory6[i + 16] = arrayEEPROMmemory0[i + 16];
      arrayEEPROMmemory6[i + 24] = arrayEEPROMmemory0[i + 24];
      arrayEEPROMmemory6[32] =   arrayEEPROMmemory0[32];
      arrayEEPROMmemory6[33] =   arrayEEPROMmemory0[33];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }  
    }
  }

  if ( arrayDP3T[3] == 1 && arrayTACTS[7] == 1 && arrayDP3T[7] == 1 && arrayDP3Tprevious[7] == 0  ) {  // ACTIVE SEQUENCE SAVE TO EEPROM //////////////////////////
    arrayDP3Tprevious[7] = 1;
    for ( byte i = 0; i < 8; i ++ ) {
      EEPROM.write(280 + i, arrayEEPROMmemory0[i]);   // 280 is the first address of arrayEEPROMmemory7
      EEPROM.write(288 + i, arrayEEPROMmemory0[i + 8]);
      EEPROM.write(296 + i, arrayEEPROMmemory0[i + 16]);
      EEPROM.write(304 + i, arrayEEPROMmemory0[i + 24]);
      EEPROM.write(312,     arrayEEPROMmemory0[32]);
      EEPROM.write(313,     arrayEEPROMmemory0[33]);
      arrayEEPROMmemory7[i] = arrayEEPROMmemory0[i];
      arrayEEPROMmemory7[i + 8] =  arrayEEPROMmemory0[i + 8];
      arrayEEPROMmemory7[i + 16] = arrayEEPROMmemory0[i + 16];
      arrayEEPROMmemory7[i + 24] = arrayEEPROMmemory0[i + 24];
      arrayEEPROMmemory7[32] =   arrayEEPROMmemory0[32];
      arrayEEPROMmemory7[33] =   arrayEEPROMmemory0[33];
      if ( i < 4 ) {
        lcd.setCursor(i * 4, 0);
        lcd.print("    ");
        lcd.setCursor(i * 4, 0);
        lcd.print(arrayEEPROMmemory0[i]);
      }
      if ( i >= 4 && i < 8 ) {
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print("    ");
        lcd.setCursor((i - 4) * 4, 1);
        lcd.print(arrayEEPROMmemory0[i]);
      }  
    }
  }


  ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////// PLAY MODE (4) ///////////////////////////////////////////////////////////////////////////

  if ( arrayDP3T[4] == 1 && arrayDP3Tprevious[4] == 0 ) {
    arrayDP3Tprevious[4] = 1;
    for ( byte i = 0; i < 8; i++ ) {
      lcd.setCursor(i * 2, 0);
      lcd.print("  ");
      lcd.setCursor(i * 2, 0);
      lcd.print(arrayDISPLAY1[i]);
      lcd.setCursor(i * 2, 1);
      lcd.print("  ");
      lcd.setCursor(i * 2, 1);
      lcd.write(byte(arrayDISPLAY2[i]));

      if (arraySKIPglide[i] == -3 || arraySKIPglide[i] == 5 ) {       // RATCHET (GATE/4)
        lcd.setCursor((i * 2) + 1, 1);
        lcd.print(" ");
        lcd.setCursor((i * 2) + 1, 1);
        lcd.write(byte(7));
      }
      if (arraySKIPglide[i] == -2 || arraySKIPglide[i] == 4 ) {       // RATCHET (GATE/3)
        lcd.setCursor((i * 2) + 1, 1);
        lcd.print(" ");
        lcd.setCursor((i * 2) + 1, 1);
        lcd.write(byte(6));
      }
      if (arraySKIPglide[i] == -1 || arraySKIPglide[i] == 3 ) {       // RATCHET (GATE/2)
        lcd.setCursor((i * 2) + 1, 1);
        lcd.print(" ");
        lcd.setCursor((i * 2) + 1, 1);
        lcd.write(byte(5));
      }
      if (arraySKIPglide[i] == 0 ) {       // DEACTIVATED SKIP and SLIDE
        lcd.setCursor((i * 2) + 1, 0);
        lcd.print(" ");
        lcd.setCursor((i * 2) + 1, 1);
        lcd.print(" ");
      }
      if (arraySKIPglide[i] == 1 || arraySKIPglide[i] >= 3 ) {       // Single click​ ​- activates the pitch GLIDE for the selected stage. The LED will light as solid green to indicate this is active.         'G' for GLIDE
        lcd.setCursor((i * 2) + 1, 0);
        lcd.print("G");
      }
      if (arraySKIPglide[i] == 2 ) {       // Double click - activates SKIP fro the selected stage. The LED will blink to indicate this is active.  'S' for SKIP
        lcd.setCursor((i * 2) + 1, 0);
        lcd.print("S");
        lcd.setCursor((i * 2) + 1, 1);
        lcd.print(" ");
      }
    }
  }

  //////////////////////

  if ( arrayDP3T[4] == 1 ) {

    if ( arrayDISPLAY1previous[count] != arrayDISPLAY1[count] ) {
      lcd.setCursor(count * 2, 0);
      lcd.print(arrayDISPLAY1[count]);
      arrayDISPLAY1previous[count] = arrayDISPLAY1[count];
    }

    if ( arrayDISPLAY2previous[count] != arrayDISPLAY2[count] ) {
      lcd.setCursor(count * 2, 1);
      lcd.write(byte(arrayDISPLAY2[count]));
      arrayDISPLAY2previous[count] = arrayDISPLAY2[count];
    }

    if ( arraySKIPglide[count] != arraySKIPglidePREVIOUS[count] || arrayDISPLAY4previous[count] != arrayDISPLAY4[count] ) {   ///////////////////// review this if statement
      arraySKIPglidePREVIOUS[count] = arraySKIPglide[count];
      arrayDISPLAY4previous[count] = arrayDISPLAY4[count];

      if ( arraySKIPglide[count] == -3 || arraySKIPglide[count] == 5 ) {       // RATCHET (GATE/4)
        lcd.setCursor((count * 2) + 1, 1);
        lcd.print(" ");
        lcd.setCursor((count * 2) + 1, 1);
        lcd.write(byte(7));
      }
      if ( arraySKIPglide[count] == -2 || arraySKIPglide[count] == 4 ) {       // RATCHET (GATE/3)
        lcd.setCursor((count * 2) + 1, 1);
        lcd.print(" ");
        lcd.setCursor((count * 2) + 1, 1);
        lcd.write(byte(6));
      }
      if ( arraySKIPglide[count] == -1 || arraySKIPglide[count] == 3 ) {       // RATCHET (GATE/2)
        lcd.setCursor((count * 2) + 1, 1);
        lcd.print(" ");
        lcd.setCursor((count * 2) + 1, 1);
        lcd.write(byte(5));
      }
      if ( arraySKIPglide[count] == 0 ) {       // DEACTIVATED SKIP and SLIDE
        lcd.setCursor((count * 2) + 1, 0);
        lcd.print(" ");
        lcd.setCursor((count * 2) + 1, 1);
        lcd.print(" ");
      }
      if ( arraySKIPglide[count] == 1 || arraySKIPglide[count] >= 3 ) {       // Single click​ ​- activates the pitch GLIDE for the selected stage. The LED will light as solid green to indicate this is active.         'G' for GLIDE
        lcd.setCursor((count * 2) + 1, 0);
        lcd.print("G");
      }
      if ( arraySKIPglide[count] == 2 ) {       // Double click - activates SKIP fro the selected stage. The LED will blink to indicate this is active.  'S' for SKIP
        lcd.setCursor((count * 2) + 1, 0);
        lcd.print("S");
        lcd.setCursor((count * 2) + 1, 1);
        lcd.print(" ");
      }
    }
  }

  //
  ///////////////////////////////////////////////////////// arrayDP3T[4] PLAY && arrayDP3T[0] PULSE COUNT /////////////////////////////////////////////////////////////////////
  //

  if ( arrayDP3T[4] == 1 && arrayDP3T[0] ==  1 ) {
    if ( arrayDP3Tprevious[6] == 0) {
      if ( arrayDP3T[6] == 1 && arrayTACTS[count] == 1 && arrayPULSEcount[count] < 8) {   //////////////////////////////////////////////////// increasing the PULSE COUNT array
        arrayPULSEcount[count] = arrayPULSEcount[count] + 1;
        arrayDISPLAY1[count] = arrayPULSEcount[count];
        arrayDP3Tprevious[6] = 1;
      }
    }
    if ( arrayDP3Tprevious[7] == 0) {
      if ( arrayDP3T[7] == 1 && arrayTACTS[count] == 1 && arrayPULSEcount[count] > 1 ) {  ///////// decreasing the PULSE COUNT array
        arrayPULSEcount[count] = arrayPULSEcount[count] - 1;
        arrayDISPLAY1[count] = arrayPULSEcount[count];
        arrayDP3Tprevious[7] = 1;
      }
    }
  }

  //
  ///////////////////////////////////////////////////////// arrayDP3T[4] PLAY && arrayDP3T[1] GATE MODE ///////////////////////////////////////////////////////////////////////
  //

  if ( arrayDP3T[4] == 1 && arrayDP3T[1] ==  1 ) {
    if ( arrayDP3Tprevious[6] == 0) {
      if ( arrayDP3T[6] == 1 && arrayTACTS[count] == 1 && arrayGATEmode[count] < 3 ) {  //////////////////////////////////////////////////////// decreasing the GATE MODE array
        arrayGATEmode[count] = arrayGATEmode[count] + 1;
        arrayDISPLAY2[count] = arrayGATEmode[count];
        arrayDP3Tprevious[6] = 1;
      }
    }
    if ( arrayDP3Tprevious[7] == 0) {
      if ( arrayDP3T[7] == 1 && arrayTACTS[count] == 1 && arrayGATEmode[count] > 0 ) {  //////////////////////////////////////////////////////// decreasing the GATE MODE array
        arrayGATEmode[count] = arrayGATEmode[count] - 1;
        arrayDISPLAY2[count] = arrayGATEmode[count];
        arrayDP3Tprevious[7] = 1;
      }
    }
  }

  //
  ///////////////////////////////////////////////////////// arrayDP3T[4] PLAY && arrayDP3T[2] SLIP / GLIDE / RATCHET //////////////////////////////////////////////////////////
  //

  if ( arrayDP3T[4] == 1 && arrayDP3T[2] == 1 ) {
    if ( arrayDP3Tprevious[6] == 0) {
      if ( arrayDP3T[6] == 1 && arrayTACTS[count] == 1 && arraySKIPglide[count] < 5 ) {   /////////////////////////////////////////////////// increasing the SKIP / SLIDE array
        arraySKIPglide[count] = arraySKIPglide[count] + 1;
        arrayDISPLAY4[count] = arraySKIPglide[count];
        arrayDP3Tprevious[6] = 1;
      }
    }
    if ( arrayDP3Tprevious[7] == 0 ) {
      if ( arrayDP3T[7] == 1 && arrayTACTS[count] == 1 && arraySKIPglide[count] > -3 ) {  /////////////////////////////////////////////////// decreasing the SKIP / SLIDE array
        arraySKIPglide[count] = arraySKIPglide[count] - 1;
        arrayDISPLAY4[count] = arraySKIPglide[count];
        arrayDP3Tprevious[7] = 1;
      }
    }
  }

  //
  ///////////////////////////////////////////////////////// arrayDP3T[5] MENU  /////////////////////////////////////////////////////////////////////////////////////////////////
  //
  //        ________________         ________________         ________________        ________________
  //       |TEMPO: 120 BPM ^|       |SCALE: CHROMA. ^|       |MIDI ROOT:  20 ^|      |START SEQ. (7) ^|
  //       |GATE WIDTH: 50%v|       |GLIDE RANGE: 2 v|       |MIDI CH.:   10 v|      |STOP SEQ.  (8) v|
  //       |________________|       |________________|       |________________|      |________________|
  //                                ^^^^^^^^^^^^^^^^^^^^^NOT IMPLEMENTED YET^^^^^^^^^^^^^^^^^^^^^^^^^^^
  //
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //

  if ( arrayDP3T[5] == 1 && MENUdisplay == 1 ) {
    if ( MENUdisplayPREVIOUS != 1 ) {
      MENUdisplayPREVIOUS = MENUdisplay;

      lcd.setCursor(0, 0);
      lcd.print("TEMPO:     ");
      lcd.setCursor(7, 0);
      lcd.print(TEMPOvalue);
      lcd.setCursor(11, 0);
      lcd.print("BPM  ");

      lcd.setCursor(0, 1);
      lcd.print("GATE WIDTH:   ");
      lcd.setCursor(11, 1);
      lcd.print(GATEwidthPERCENT);
      lcd.setCursor(14, 1);
      lcd.print("% ");
    }

    if ( TEMPOvaluePREVIOUS != TEMPOvalue ) {
      lcd.setCursor(0, 0);
      lcd.print("TEMPO:     ");
      lcd.setCursor(7, 0);
      lcd.print(TEMPOvalue);
      lcd.setCursor(11, 0);
      lcd.print("BPM  ");
      TEMPOvaluePREVIOUS = TEMPOvalue;
    }

    if ( GATEwidthPERCENTprevious != GATEwidthPERCENT ) {
      lcd.setCursor(0, 1);
      lcd.print("GATE WIDTH:   ");
      lcd.setCursor(11, 1);
      lcd.print(GATEwidthPERCENT);
      lcd.setCursor(14, 1);
      lcd.print("% ");
      GATEwidthPERCENTprevious = GATEwidthPERCENT;
    }
  }

  if ( arrayDP3Tprevious[6] == 0) {
    if ( arrayDP3T[5] == 1 && arrayDP3T[6] == 1 && arrayTACTS[0] == 1 && TEMPOvalue < 200 ) {                              ////////////////////////// increasing the TEMPO by 1
      TEMPOvaluePREVIOUS = TEMPOvalue;
      TEMPOvalue = TEMPOvalue + 1;
      arrayDP3Tprevious[6] = 1;
    }
    if ( arrayDP3T[5] == 1 && arrayDP3T[6] == 1 && arrayTACTS[1] == 1 && GATEwidthPERCENT < 95 ) {             //////////////////// increasing the GATE lenght by 1
      GATEwidthPERCENTprevious = GATEwidthPERCENT;
      GATEwidthPERCENT = GATEwidthPERCENT + 1;
      arrayDP3Tprevious[6] = 1;
    }
  }

  if ( arrayDP3Tprevious[7] == 0) {
    if ( arrayDP3T[5] == 1 && arrayDP3T[7] == 1 && arrayTACTS[0] == 1 && TEMPOvalue > 20 ) {                               /////////////////////////// decreasing the TEMPO by 1
      TEMPOvaluePREVIOUS = TEMPOvalue;
      TEMPOvalue = TEMPOvalue - 1;
      arrayDP3Tprevious[7] = 1;
    }
    if ( arrayDP3T[5] == 1 && arrayDP3T[7] == 1 && arrayTACTS[1] == 1 && GATEwidthPERCENT > 5 ) {               ////////////////////// decreasing the GATE lenght by 1
      GATEwidthPERCENTprevious = GATEwidthPERCENT;
      GATEwidthPERCENT = GATEwidthPERCENT - 1;
      arrayDP3Tprevious[7] = 1;
    }
  }

  ////////////////////// MENU (5) and SCALE AND GLIDE RANGE SUB MENU (2) ////////////////////////////////////////// ////////////

  if ( arrayDP3T[5] == 1 && MENUdisplay == 2 ) {
    if ( MENUdisplayPREVIOUS != 2 ) {
      MENUdisplayPREVIOUS = MENUdisplay;

      lcd.setCursor(0, 0);
      lcd.print("SCALE: CHROMA.  ");

      lcd.setCursor(0, 1);
      lcd.print("GLIDE RANGE:    ");
      lcd.setCursor(13, 1);
      lcd.print(GLIDErange);
    }

    if ( GLIDErangePREVIOUS != GLIDErange ) {
      lcd.setCursor(0, 1);
      lcd.print("GLIDE RANGE:    ");
      lcd.setCursor(13, 1);
      lcd.print(GLIDErange);
      GLIDErangePREVIOUS = GLIDErange;
    }
  }

  if ( arrayDP3Tprevious[6] == 0 ) {
    if ( arrayDP3T[5] == 1 && arrayDP3T[6] == 1 && arrayTACTS[3] == 1 && GLIDErange < 4 ) {
      GLIDErange = GLIDErange + 1;
      arrayDP3Tprevious[6] = 1;
    }
  }

  if ( arrayDP3Tprevious[7] == 0 ) {
    if ( arrayDP3T[5] == 1 && arrayDP3T[7] == 1 && arrayTACTS[3] == 1 && GLIDErange > 1 ) {
      GLIDErangePREVIOUS = GLIDErange;
      GLIDErange = GLIDErange - 1;
      arrayDP3Tprevious[7] = 1;
    }
  }



  ///////////////////////////////////////////////////////////////////////





  if ( arrayDP3T[3] == 0 ) {  // happens only once when switched off SAVE/LOAD MODE
    FLAG1 = 0;
  }


  if ( arrayDP3Tprevious[6] == 0 && arrayDP3Tprevious[7] == 0 ) { // RESET MIDI WHEN BOTH UP AND DOWN BUTTONS PRESSED
    if ( arrayDP3T[6] == 1 && arrayDP3T[7] == 1 ) {
      uClock.stop();
      ControlChange (0xB0, 0x7B, 0x00); // Bn 7B 00 All Notes Off
      delay (10);
      ControlChange (0xB0, 0x78, 0x00); // B0 78 00 All Sounds Off
      delay (10);
      ControlChange (0xB0, 0x79, 0x7F); // Bn 79 7F Reset All Controllers
      delay (10);
      ProgramChange(192, 25);
      delay (10);
      uClock.start();
    }
  }

  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  if ( arrayTACTS[0] == 0 ) {
    arrayTACTSprevious[0] = 0;
  }
  if ( arrayTACTS[1] == 0 ) {
    arrayTACTSprevious[1] = 0;
  }
  if ( arrayTACTS[2] == 0 ) {
    arrayTACTSprevious[2] = 0;
  }
  if ( arrayTACTS[3] == 0 ) {
    arrayTACTSprevious[3] = 0;
  }
  if ( arrayTACTS[4] == 0 ) {
    arrayTACTSprevious[4] = 0;
  }
  if ( arrayTACTS[5] == 0 ) {
    arrayTACTSprevious[5] = 0;
  }
  if ( arrayTACTS[6] == 0 ) {
    arrayTACTSprevious[6] = 0;
  }
  if ( arrayTACTS[7] == 0 ) {
    arrayTACTSprevious[7] = 0;
  }

  if ( arrayDP3T[0] == 0 ) {
    arrayDP3Tprevious[0] = 0;
  }
  if ( arrayDP3T[1] == 0 ) {
    arrayDP3Tprevious[1] = 0;
  }
  if ( arrayDP3T[2] == 0 ) {
    arrayDP3Tprevious[2] = 0;
  }
  if ( arrayDP3T[3] == 0 ) {
    arrayDP3Tprevious[3] = 0;
  }
  if ( arrayDP3T[4] == 0 ) {
    arrayDP3Tprevious[4] = 0;
  }
  if ( arrayDP3T[5] == 0 ) {
    MENUdisplay = 1;
    MENUdisplayPREVIOUS = 0;
    arrayDP3Tprevious[5] = 0;
  }
  if ( arrayDP3T[6] == 0 ) {
    arrayDP3Tprevious[6] = 0;
  }
  if ( arrayDP3T[7] == 0 ) {
    arrayDP3Tprevious[7] = 0;
  }

}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
  0x00: bank select (0 is default, 0x78 and 0x7f is drums, 0x79 melodic)
  0x06: RPN MSB: 0 = bend range, 2 = coarse tune
  0x07: channel volume
  0x0a: pan control
  0x0b: expression (changes volume)
  0x0c: effect control 1 (sets global reverb decay)
  0x26: RPN LSB: 0 = bend range
  0x40: hold1
  0x42: sustenuto
  0x5b: effects level (channel reverb level)
  0x62,0x63,0x64,0x65: NRPN and RPN selects
  0x78: all sound off
  0x79: reset all controllers
  0x7b, 0x7c, 0x7d: all notes off
*/
